/*  cURL:
CURLOPT_FAILONERROR:
https://curl.se/libcurl/c/CURLOPT_FAILONERROR.html
https://curl.se/libcurl/c/libcurl-errors.html#CURLEHTTPRETURNEDERROR
*/

#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"
#include <curl/curl.h>
#include <sstream>
#include <string>



using namespace std;
struct Input
{
    vector<double> numbers;
    size_t bin_count{};
};

Input input_data(istream& inx, bool prompt)
{
    size_t number_count;
    if (prompt) cerr << "Enter number count: ";
    inx >> number_count;


    Input in;
    in.numbers.resize(number_count);

    vector<double> numbers(number_count);
    if (prompt) cerr << "Enter numbers: ";
    for (int i = 0; i < number_count; i++)
    {
        inx >> in.numbers[i];
    }

    if (prompt) cerr << "Enter bin count: ";
    inx >> in.bin_count;
    return in;
}


size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx)
{
    // TODO:    .
    size_t data_size = item_size * item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

Input
download(const string& address)
{
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if (curl)
    {
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());

    // ----- 17
        curl_easy_setopt(curl, CURLOPT_FAILONERROR, 1L); // HTTP- ( 4xx  5xx)   cURL
    //-------

        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);

        CURLcode res = curl_easy_perform(curl);
        if (res != CURLE_OK)
        {
            // ----- 17
           if (res == CURLE_HTTP_RETURNED_ERROR) {
                cerr << "HTTP Error: Server returned 4xx/5xx status code for URL: " << address << endl; //  ,    
            } else {
                cerr << "cURL error (" << res << "): " << curl_easy_strerror(res) << endl;
            }
            //--------
            curl_easy_cleanup(curl);
            exit(1);
        }
        curl_easy_cleanup(curl);
    }
    else
    {
        cerr << "curl_easy_init() failed" << endl;
        exit(1);
    }

    return input_data(buffer, false);
}

int main(int argc, char* argv[])
{
    Input input;
    if (argc > 1)
    {
        input = download(argv[1]);
    }
    else
    {
        input = input_data(cin, true);
    }
    auto bins = make_histogram(input.numbers, input.bin_count);
    //show_histogram_text(bins, in.bin_count, bin_size);
    show_histogram_svg(bins);
}

